

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "readbuf.h"

#define _V_END 0
#define _RS_VERSION "0.1.3"
#define _RS_ADD_SIBLING 1
#define _RS_ADD_DOWN 2
#define _BUFFER_SIZE 10240

char *RB_front, *RB_back, *RB_start, *RB_stop, *RB_tmp;
int RB_size;
int RB_waterlevel = 0;
int RS_length = 0;


/* Our file-read buffer */
char _databuffer[_BUFFER_SIZE];
int _RS_FB_highmark = 0;
int _RS_FB_lowmark = 0;
int _RS_FB_eof = 0;
FILE *_RS_FB_stream = NULL;

/* Operational flags */
int verbose = 0;
int fastexit;
int fastscan;
char *currentfilename = NULL;


/*-----------------------------------------------
 * RS_init_FB()
 *
 * Initialises and cleans the file buffer */
int RS_FB_init( void ){
	_RS_FB_highmark = -1;
	_RS_FB_lowmark = 0;
	_RS_FB_eof = 0;	
	return 0;
	}	

/*------------------------------------------------
 * RS_FB_setstream
 *
 * Sets the stream which the file-buffer routines
 * should be using to get more data from */
int RS_FB_setstream( FILE *f ){
	
	_RS_FB_stream = f;
	RS_FB_init();
	
	return 0;
	}
	
	
/*------------------------------------------------
 * RS_FB_feof
 *
 * Just like the normal feof() call, except it 
 * returns the status of the FileBuffer stream */
int RS_FB_feof( void ){
	
	/* if we've had the EOF flag marked from a previous
	 * RS_FB_getblock() call, then we just check to see
	 * if our data has run out yet from this last block */
	if (_RS_FB_eof && (_RS_FB_lowmark > _RS_FB_highmark))
		return 1;
	else return 0;
	}

/*------------------------------------------------
 * RS_FB_getblock
 *
 * Gets a _BLOCK_SIZE data block from the stream as
 * specified in _RS_FB_stream.  If the stream ends
 * then we set our _RS_FB_eof flag in here */	
int RS_FB_getblock( void ){
	
	size_t bytesread;

	/* try read in _BUFFER_SIZE bytes from the stream */
	bytesread = fread( _databuffer, sizeof(char), _BUFFER_SIZE, _RS_FB_stream);

	/* if we didn't get all of what we asked, for, then we have
	 * reached the end of the file, hence mark the EOF flag */
	if (bytesread != _BUFFER_SIZE) {
		_RS_FB_eof = 1;	
		}

	/* Now that we have a full tank of "data", set the high/low
	 * marks back to their "full" values */
	_RS_FB_highmark = bytesread -1;
	_RS_FB_lowmark = 0;	
	
	/* as a courtosy, return the number of bytes read just incase
	 * we want to do something from the calling function */
	return bytesread;
	}
	
	
/*------------------------------------------------
 * RS_FB_getbyte
 *
 * sets the suppled 'c' char with the next byte in
 * the filebuffer.  NOTE - RS_FB_eof() should be 
 * called before this function, because we cannot
 * give a suitable response back via the char value */
int RS_FB_getbyte( char *c ){
	
	/* if we've run out of buffer space... */
	if (_RS_FB_highmark < _RS_FB_lowmark) {
		RS_FB_getblock();
		}
		
	/* return the next char */
	*c = _databuffer[_RS_FB_lowmark];
	
	/* increment our buffer mark */
	_RS_FB_lowmark++;
	
	return 1;
	}
	
	
	
int RS_FB_getline( char *line, int max, FILE *f ){
	
	char *front, *back;
	
	while(!RS_FB_feof()){
		RS_FB_getchar(line);
		if (line == '\n') break;
		if (line == '\0') break;
		line++;
		}

	return 0;
	}
	
int RS_FB_unget( void ){
	
	_RS_FB_lowmark--;
	}
		
	
		
/*------------------------------------------END. */